import * as React from 'react';
import * as ReactDOM from 'react-dom';
import App from './App';
import registerServiceWorker from './registerServiceWorker';
import './index.css';
import {
  DiagramEngine,
  DefaultNodeFactory,
  DefaultLinkFactory,
  DiagramModel,
  DefaultNodeModel,
  LinkModel,
  DefaultPortModel,
  DiagramWidget
} from 'storm-react-diagrams';
import { Machine } from 'xstate';
import * as utils from 'xstate/lib/graph';

const pedestrianStates = {
  initial: 'walk',
  states: {
    walk: {
      on: {
        PED_COUNTDOWN: 'wait'
      }
    },
    wait: {
      on: {
        PED_COUNTDOWN: 'stop'
      }
    },
    stop: {}
  }
};

const lightMachine = Machine({
  key: 'light',
  initial: 'green',
  states: {
    green: {
      on: {
        TIMER: 'yellow',
        POWER_OUTAGE: 'red'
      }
    },
    yellow: {
      on: {
        TIMER: 'red',
        POWER_OUTAGE: 'red'
      }
    },
    red: {
      on: {
        TIMER: 'green',
        POWER_OUTAGE: 'red'
      },
      ...pedestrianStates
    }
  }
});

const nodes = utils.getNodes(lightMachine);
const edges = utils.getEdges(lightMachine);

console.log(nodes);

// 1) setup the diagram engine
var engine = new DiagramEngine();
engine.registerNodeFactory(new DefaultNodeFactory());
engine.registerLinkFactory(new DefaultLinkFactory());

// 2) setup the diagram model
var model = new DiagramModel();
const ports = {
  in: {},
  out: {}
};

console.log(ports);

// 3-A) create a default node
nodes.forEach((node, i) => {
  var node1 = new DefaultNodeModel(node.id, 'rgb(0,192,255)');
  ports.in[node.id] = node1.addPort(new DefaultPortModel(true, 'in'));
  ports.out[node.id] = node1.addPort(new DefaultPortModel(false, 'out'));
  node1.x = 100 * i;
  node1.y = 100;

  model.addNode(node1);
});

edges.forEach(edge => {
  var link1 = new LinkModel();
  link1.setSourcePort(ports.out[edge.source.id]);
  link1.setTargetPort(ports.in[edge.target.id]);
  model.addLink(link1);
});

// 5) load model into engine
engine.setDiagramModel(model);

ReactDOM.render(
  <DiagramWidget diagramEngine={engine} />,
  document.getElementById('root') as HTMLElement
);
registerServiceWorker();
