"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
function getNodes(node) {
    var states = node.states;
    var nodes = Object.keys(states).reduce(function (accNodes, stateKey) {
        var subState = states[stateKey];
        var subNodes = getNodes(states[stateKey]);
        accNodes.push.apply(accNodes, [subState].concat(subNodes));
        return accNodes;
    }, []);
    return nodes;
}
exports.getNodes = getNodes;
function getTransitionStateKeys(transition) {
    if (typeof transition === 'string') {
        return [transition];
    }
    return Object.keys(transition);
}
function getEdges(node, visited) {
    if (visited === void 0) { visited = {}; }
    var states = node.states;
    visited[node.key] = true;
    var subNodeEdges = Object.keys(states).reduce(function (_edges, key) {
        if (visited[key]) {
            return _edges;
        }
        var subState = states[key];
        _edges.push.apply(_edges, getEdges(subState, visited));
        visited[key] = true;
        return _edges;
    }, []);
    if (!node.on) {
        return subNodeEdges;
    }
    var edges = Object.keys(node.on).reduce(function (accEdges, event) {
        if (!node.on || !node.parent) {
            return accEdges;
        }
        var parent = node.parent;
        var transition = node.on[event];
        if (!transition) {
            return accEdges;
        }
        var subStateKeys = getTransitionStateKeys(transition);
        subStateKeys.forEach(function (subStateKey) {
            var subNode = parent.getState(subStateKey);
            var edge = { event: event, source: node, target: subNode };
            accEdges.push(edge);
            if (!visited[subStateKey]) {
                accEdges.push.apply(accEdges, getEdges(subNode, visited));
                visited[subStateKey] = true;
            }
        });
        return accEdges;
    }, []);
    return subNodeEdges.concat(edges);
}
exports.getEdges = getEdges;
function getAdjacencyMap(node) {
    var eventMap = node.parent
        ? {}
        : undefined;
    var adjacency = {};
    if (node.on) {
        for (var _i = 0, _a = Object.keys(node.on); _i < _a.length; _i++) {
            var event_1 = _a[_i];
            if (!node.parent || !eventMap) {
                continue;
            }
            var nextState = node.machine.transition(node.relativeId, event_1);
            if (!nextState) {
                continue;
            }
            eventMap[event_1] = { state: nextState.value };
            // const transitionConfig = node.on[event];
            // const subStateKeys = getTransitionStateKeys(transitionConfig);
            // if (!subStateKeys.length) {
            //   continue;
            // }
            // // for non-conditional adjacency maps, just return first substate
            // const nextState = node.parent.getState(subStateKeys[0]) as StateNode;
            // let nextStateId = nextState.id;
            // if (nextState.initial) {
            //   nextStateId += '.' + nextState.initial;
            // }
            // eventMap[event] = nextStateId;
        }
    }
    if (eventMap) {
        adjacency[node.relativeId] = eventMap;
    }
    if (node.states) {
        for (var _b = 0, _c = Object.keys(node.states); _b < _c.length; _b++) {
            var stateKey = _c[_b];
            var state = node.states[stateKey];
            var stateAdjacency = utils_1.mapValues(getAdjacencyMap(state), function (value) {
                return __assign({}, eventMap, value);
            });
            Object.assign(adjacency, stateAdjacency);
        }
    }
    return adjacency;
}
exports.getAdjacencyMap = getAdjacencyMap;
function getShortestPaths(machine) {
    if (!machine.states || !machine.initial) {
        return undefined;
    }
    var adjacency = getAdjacencyMap(machine);
    var initialId = trieToString(machine.initialState);
    var pathMap = (_a = {},
        _a[initialId] = [],
        _a);
    var visited = new Set();
    function util(stateId) {
        visited.add(stateId);
        var eventMap = adjacency[stateId];
        for (var _i = 0, _a = Object.keys(eventMap); _i < _a.length; _i++) {
            var event_2 = _a[_i];
            var nextStateValue = eventMap[event_2].state;
            if (!nextStateValue) {
                continue;
            }
            var nextStateId = trieToString(nextStateValue);
            if (!pathMap[nextStateId] ||
                pathMap[nextStateId].length > pathMap[stateId].length + 1) {
                pathMap[nextStateId] = pathMap[stateId].concat([{ state: stateId, event: event_2 }]);
            }
        }
        for (var _b = 0, _c = Object.keys(eventMap); _b < _c.length; _b++) {
            var event_3 = _c[_b];
            var nextStateValue = eventMap[event_3].state;
            if (!nextStateValue) {
                continue;
            }
            var nextStateId = trieToString(nextStateValue);
            if (visited.has(nextStateId)) {
                continue;
            }
            util(nextStateId);
        }
        return pathMap;
    }
    util(initialId);
    return pathMap;
    var _a;
}
exports.getShortestPaths = getShortestPaths;
function trieToString(trie) {
    if (typeof trie === 'string') {
        return trie;
    }
    var firstKey = Object.keys(trie)[0];
    return [firstKey].concat(trieToString(trie[firstKey])).join('.');
}
function getSimplePaths(machine) {
    if (!machine.states || !machine.initial) {
        return undefined;
    }
    var adjacency = getAdjacencyMap(machine);
    var visited = new Set();
    var path = [];
    var paths = {};
    function util(fromPathId, toPathId) {
        visited.add(fromPathId);
        if (fromPathId === toPathId) {
            paths[toPathId] = paths[toPathId] || [];
            paths[toPathId].push(path.slice());
        }
        else {
            for (var _i = 0, _a = Object.keys(adjacency[fromPathId]); _i < _a.length; _i++) {
                var subEvent = _a[_i];
                var nextStateValue = adjacency[fromPathId][subEvent].state;
                if (!nextStateValue) {
                    continue;
                }
                var nextStateId = trieToString(nextStateValue);
                if (!visited.has(nextStateId)) {
                    path.push({ state: fromPathId, event: subEvent });
                    util(nextStateId, toPathId);
                }
            }
        }
        path.pop();
        visited.delete(fromPathId);
    }
    var initialStateId = machine.initialState;
    Object.keys(adjacency).forEach(function (nextStateId) {
        util(initialStateId, nextStateId);
    });
    return paths;
}
exports.getSimplePaths = getSimplePaths;
