import { Event, StateValue, Transition, Action, Machine, StandardMachine, ParallelMachine, StateOrMachineConfig, MachineConfig, ParallelMachineConfig, EventType } from './types';
import matchesState from './matchesState';
import mapState from './mapState';
import State from './State';
declare class StateNode<TStateKey extends string = string, TEventType extends string = string> {
    key: string;
    id: string;
    relativeId: string;
    initial?: string;
    parallel?: boolean;
    states: Record<TStateKey, StateNode>;
    on?: Record<TEventType, Transition<TStateKey> | undefined>;
    onEntry?: Action;
    onExit?: Action;
    strict: boolean;
    parent?: StateNode;
    machine: StateNode;
    private __cache;
    constructor(config: StateOrMachineConfig<TStateKey, TEventType>);
    getStateNodes(state: StateValue | State): StateNode[];
    transition(state: StateValue | State, event: Event, extendedState?: any): State | undefined;
    maybeTransition(state: StateValue | State, event: Event, extendedState?: any): State;
    private transitionStateValue(state, event, extendedState?);
    private next(event, history?, extendedState?);
    readonly initialState: StateValue | undefined;
    getStates(stateValue: StateValue): StateNode[];
    getState(relativeStateId: string | string[]): StateNode | undefined;
    readonly events: EventType[];
    getRelativeValue(toNode?: StateNode): StateValue;
    private getActions(prevStateValue, nextStateValue, actions);
}
declare function Machine(config: MachineConfig | ParallelMachineConfig): StandardMachine | ParallelMachine;
export { StateNode, Machine, State, matchesState, mapState };
